"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const messaging_1 = require("../../lib/messaging");
const NS_MEDIA = "urn:x-cast:com.google.cast.media";
class Media {
    constructor(referenceId, session) {
        this.referenceId = referenceId;
        this.session = session;
        this.session.createChannel(NS_MEDIA);
        this.channel = this.session.channelMap.get(NS_MEDIA);
        this.channel.on("message", (data) => {
            if (data && data.type === "MEDIA_STATUS"
                && data.status && data.status.length > 0) {
                const status = data.status[0];
                const messageData = {
                    _lastCurrentTime: Date.now() / 1000,
                    currentTime: status.currentTime,
                    customData: status.customData,
                    playbackRate: status.playbackRate,
                    playerState: status.playerState,
                    repeatMode: status.repeatMode
                };
                if (status.volume) {
                    messageData._volumeLevel = status.volume.level;
                    messageData._volumeMuted = status.volume.muted;
                }
                if (status.media) {
                    messageData.media = status.media;
                }
                if (status.mediaSessionId) {
                    messageData.mediaSessionId = status.mediaSessionId;
                }
                this.sendMessage("shim:/media/update", messageData);
            }
        });
    }
    messageHandler(message) {
        switch (message.subject) {
            case "bridge:/media/sendMediaMessage": {
                let error = false;
                try {
                    this.channel.send(message.data.message);
                }
                catch (err) {
                    error = true;
                }
                this.sendMessage("shim:/media/sendMediaMessageResponse", {
                    messageId: message.data.messageId,
                    error
                });
                break;
            }
        }
    }
    sendMessage(subject, data) {
        messaging_1.sendMessage({
            subject,
            data,
            _id: this.referenceId
        });
    }
}
exports.default = Media;
