"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const castv2_1 = require("castv2");
const events_1 = require("events");
const Session_1 = require("./Session");
class StatusListener extends events_1.EventEmitter {
    constructor(host, port) {
        super();
        this.client = new castv2_1.Client();
        this.client.connect({ host, port }, this.onConnect.bind(this));
        this.client.on("close", () => {
            clearInterval(this.clientHeartbeatIntervalId);
        });
        this.client.on("error", () => {
            clearInterval(this.clientHeartbeatIntervalId);
        });
    }
    deregister() {
        if (this.clientReceiver) {
            this.clientReceiver.send({ type: "CLOSE" });
        }
        this.client.close();
    }
    onConnect() {
        const sourceId = "sender-0";
        const destinationId = "receiver-0";
        const clientConnection = this.client.createChannel(sourceId, destinationId, Session_1.NS_CONNECTION, "JSON");
        const clientHeartbeat = this.client.createChannel(sourceId, destinationId, Session_1.NS_HEARTBEAT, "JSON");
        const clientReceiver = this.client.createChannel(sourceId, destinationId, Session_1.NS_RECEIVER, "JSON");
        clientReceiver.on("message", data => {
            switch (data.type) {
                case "CLOSE": {
                    this.client.close();
                    break;
                }
                case "RECEIVER_STATUS": {
                    this.emit("receiverStatus", data.status);
                    break;
                }
                case "MEDIA_STATUS": {
                    this.emit("mediaStatus", data.status);
                    break;
                }
            }
        });
        clientConnection.send({ type: "CONNECT" });
        clientHeartbeat.send({ type: "PING" });
        clientReceiver.send({ type: "GET_STATUS", requestId: 1 });
        this.clientReceiver = clientReceiver;
        this.clientHeartbeatIntervalId = setInterval(() => {
            clientHeartbeat.send({ type: "PING" });
        }, 5000);
    }
}
exports.default = StatusListener;
