"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stopDiscovery = exports.startDiscovery = void 0;
const mdns_1 = __importDefault(require("mdns"));
const StatusListener_1 = __importDefault(require("./chromecast/StatusListener"));
const messaging_1 = require("../lib/messaging");
const browser = mdns_1.default.createBrowser(mdns_1.default.tcp("googlecast"), {
    resolverSequence: [
        mdns_1.default.rst.DNSServiceResolve(),
        "DNSServiceGetAddrInfo" in mdns_1.default.dns_sd
            ? mdns_1.default.rst.DNSServiceGetAddrInfo()
            : mdns_1.default.rst.getaddrinfo({ families: [4] }),
        mdns_1.default.rst.makeAddressesUnique()
    ]
});
function onBrowserServiceUp(service) {
    messaging_1.sendMessage({
        subject: "main:/serviceUp",
        data: {
            host: service.addresses[0],
            port: service.port,
            id: service.txtRecord.id,
            friendlyName: service.txtRecord.fn
        }
    });
}
function onBrowserServiceDown(service) {
    messaging_1.sendMessage({
        subject: "main:/serviceDown",
        data: {
            id: service.txtRecord.id
        }
    });
}
browser.on("serviceUp", onBrowserServiceUp);
browser.on("serviceDown", onBrowserServiceDown);
function startDiscovery(options) {
    if (options.shouldWatchStatus) {
        browser.on("serviceUp", onStatusBrowserServiceUp);
        browser.on("serviceDown", onStatusBrowserServiceDown);
    }
    browser.start();
    const statusListeners = new Map();
    function onStatusBrowserServiceUp(service) {
        const { id } = service.txtRecord;
        const listener = new StatusListener_1.default(service.addresses[0], service.port);
        listener.on("receiverStatus", (status) => {
            const receiverStatusMessage = {
                subject: "main:/receiverStatus",
                data: {
                    id,
                    status: {
                        volume: {
                            level: status.volume.level,
                            muted: status.volume.muted
                        }
                    }
                }
            };
            if (status.applications && status.applications.length) {
                const application = status.applications[0];
                receiverStatusMessage.data.status.application = {
                    appId: application.appId,
                    displayName: application.displayName,
                    isIdleScreen: application.isIdleScreen,
                    statusText: application.statusText
                };
            }
            messaging_1.sendMessage(receiverStatusMessage);
        });
        statusListeners.set(id, listener);
    }
    function onStatusBrowserServiceDown(service) {
        const { id } = service.txtRecord;
        if (statusListeners.has(id)) {
            statusListeners.get(id).deregister();
            statusListeners.delete(id);
        }
    }
}
exports.startDiscovery = startDiscovery;
function stopDiscovery() {
    browser.stop();
}
exports.stopDiscovery = stopDiscovery;
