"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stopReceiverSelector = exports.startReceiverSelector = void 0;
const child_process_1 = __importDefault(require("child_process"));
const path_1 = __importDefault(require("path"));
const messaging_1 = require("../lib/messaging");
function fatal(message) {
    console.error(message);
    process.exit(1);
}
let selectorApp;
let selectorAppOpen = false;
function startReceiverSelector(data) {
    if (process.platform !== "darwin") {
        fatal("Invalid platform for native receiver selector.");
    }
    if (!data) {
        fatal("Missing native selector data");
    }
    else {
        try {
            JSON.parse(data);
        }
        catch (err) {
            fatal("Invalid native selector data.");
        }
    }
    if (selectorApp && selectorAppOpen) {
        selectorApp.kill();
        selectorAppOpen = false;
    }
    const selectorPath = path_1.default.join(process.cwd(), "fx_cast_selector.app/Contents/MacOS/fx_cast_selector");
    selectorApp = child_process_1.default.spawn(selectorPath, [data]);
    selectorAppOpen = true;
    if (selectorApp.stdout) {
        selectorApp.stdout.setEncoding("utf-8");
        selectorApp.stdout.on("data", data => {
            const jsonData = JSON.parse(data);
            if (!jsonData.mediaType) {
                messaging_1.sendMessage({
                    subject: "main:/receiverSelector/stop",
                    data: jsonData
                });
                return;
            }
            messaging_1.sendMessage({
                subject: "main:/receiverSelector/selected",
                data: jsonData
            });
        });
    }
    selectorApp.on("error", err => {
        messaging_1.sendMessage({
            subject: "main:/receiverSelector/error",
            data: err.message
        });
    });
    selectorApp.on("close", () => {
        if (selectorAppOpen) {
            selectorAppOpen = false;
            messaging_1.sendMessage({
                subject: "main:/receiverSelector/close"
            });
        }
    });
}
exports.startReceiverSelector = startReceiverSelector;
function stopReceiverSelector() {
    if (!(selectorApp === null || selectorApp === void 0 ? void 0 : selectorApp.killed)) {
        selectorApp === null || selectorApp === void 0 ? void 0 : selectorApp.kill();
        selectorAppOpen = false;
    }
}
exports.stopReceiverSelector = stopReceiverSelector;
