"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const messaging_1 = require("./lib/messaging");
const chromecast_1 = require("./components/chromecast");
const discovery_1 = require("./components/discovery");
const mediaServer_1 = require("./components/mediaServer");
const receiverSelector_1 = require("./components/receiverSelector");
const package_json_1 = require("../../package.json");
process.on("SIGTERM", () => {
    discovery_1.stopDiscovery();
    mediaServer_1.stopMediaServer();
    receiverSelector_1.stopReceiverSelector();
});
messaging_1.decodeTransform.on("data", (message) => {
    if (message.subject.startsWith("bridge:/session/")) {
        chromecast_1.handleSessionMessage(message);
        return;
    }
    if (message.subject.startsWith("bridge:/media/")) {
        chromecast_1.handleMediaMessage(message);
        return;
    }
    switch (message.subject) {
        case "bridge:/getInfo": {
            messaging_1.encodeTransform.write(package_json_1.__applicationVersion);
            break;
        }
        case "bridge:/initialize": {
            discovery_1.startDiscovery(message.data);
            break;
        }
        case "bridge:/stopReceiverApp": {
            chromecast_1.stopReceiverApp(message.data.receiver.host, message.data.receiver.port);
            break;
        }
        case "bridge:/receiverSelector/open": {
            receiverSelector_1.startReceiverSelector(message.data);
            break;
        }
        case "bridge:/receiverSelector/close": {
            receiverSelector_1.stopReceiverSelector();
            break;
        }
        case "bridge:/mediaServer/start": {
            mediaServer_1.startMediaServer(message.data.filePath, message.data.port);
            break;
        }
        case "bridge:/mediaServer/stop": {
            mediaServer_1.stopMediaServer();
            break;
        }
    }
});
