"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = void 0;
const child_process_1 = require("child_process");
const stream_1 = require("stream");
const ws_1 = __importDefault(require("ws"));
const transforms_1 = require("./transforms");
function init(port) {
    process.stdout.write("Starting WebSocket server... ");
    const wss = new ws_1.default.Server({ port }, () => {
        console.log("Done!");
    });
    wss.on("error", (err) => {
        console.log("Failed!");
        console.error(err);
    });
    wss.on("connection", socket => {
        const messageStream = new stream_1.Readable({ objectMode: true });
        messageStream._read = () => { };
        socket.on("message", (message) => {
            messageStream.push(JSON.parse(message));
        });
        const bridge = child_process_1.spawn(process.execPath, [process.argv[1]]);
        messageStream
            .pipe(new transforms_1.EncodeTransform())
            .pipe(bridge.stdin);
        bridge.stdout
            .pipe(new transforms_1.DecodeTransform())
            .on("data", data => {
            if (socket.readyState === ws_1.default.OPEN) {
                socket.send(JSON.stringify(data));
            }
        });
        socket.on("close", () => bridge.kill());
        bridge.on("exit", () => socket.close());
    });
}
exports.init = init;
